<?php

/**
 * @file
 * Provide admin functions to alias_lock.
 */

/**
 * Fetch a specific URL alias from the locked table.
 *
 * @param array|int|string $conditions
 *   A string representing the source, a number representing the pid, or an
 *   array of query conditions.
 *
 * @return array|false
 *   FALSE if no alias was found or an associative array containing the
 *   following keys:
 *   - source: The internal system path.
 *   - alias: The URL alias.
 *   - pid: Unique path alias identifier.
 *   - language: The language of the alias.
 */
function alias_lock_load($conditions) {
  if (is_numeric($conditions)) {
    $conditions = array('pid' => (int) $conditions);
  }
  else {
    if (is_string($conditions)) {
      $conditions = array('source' => $conditions);
    }
  }
  if (!is_array($conditions)) {
    return FALSE;
  }

  if (empty($conditions['pid'])) {
    $path = path_load($conditions);
    $conditions['pid'] = (int) $path['pid'];
  }

  $aliases = &drupal_static(__FUNCTION__, array());
  $static_key = $conditions['pid'];
  if (!array_key_exists($static_key, $aliases)) {
    $data = db_select('alias_lock')
      ->condition('pid', $conditions['pid'])
      ->fields('alias_lock')
      ->execute()
      ->fetchAssoc();
    if (!empty($data)) {
      if (($path = path_load(array('pid' => $conditions['pid'])))) {
        $data += $path;
      }
      $data['#alias_locked'] = TRUE;
    }
    $aliases[$static_key] = $data;
  }

  return $aliases[$static_key];
}

/**
 * Batch restore all locked aliases.
 *
 * NOTE: The pid will change, will be different on restored aliases.
 *
 * This can be run programatically at any time and any locked aliases that are
 * not present in the url_alias table will be restored.
 */
function alias_lock_batch_restore() {
  $query = db_query("SELECT * FROM {alias_lock} WHERE pid NOT in (SELECT pid FROM {url_alias})");
  $query->execute();
  $aliases = array();
  while ($locked = $query->fetchAssoc()) {

    // Restore the original path, but the pid will change.
    $path = $locked;
    $path += array('language' => LANGUAGE_NONE);
    unset($path['pid']);
    drupal_write_record('url_alias', $path);

    // Update the locked version with the new pid.
    db_update('alias_lock')
      ->fields(array('pid' => $path['pid']))
      ->condition('pid', $locked['pid'])
      ->execute();

    $path['original'] = $locked;
    module_invoke_all('alias_lock_restore', $path);

    // Clear internal properties.
    unset($path['original']);

    // Clear the static alias cache.
    drupal_clear_path_cache($path['source']);

    $aliases[] = $path['alias'];
  }
  unset($_SESSION['alias_lock_do_batch']);
  if (count($aliases)) {
    drupal_set_message(t("The following aliases were not deleted since they are locked: %aliases", array(
      '%aliases' => implode(', ', $aliases),
    )), 'status', FALSE);
  }
}

/**
 * Lock an alias.
 *
 * @param array $path
 *   An associative array containing the following keys:
 *   - source: The internal system path.
 *   - alias: The URL alias.
 *   - pid: (optional) Unique path alias identifier.
 *   - language: (optional) The language of the alias.
 *
 * @return null|ALIAS_LOCK_LOCKED
 *   ALIAS_LOCK_LOCKED is returned if the alias was previously unlocked and is
 *   now locked.
 */
function alias_lock_lock(array $path) {
  if (empty($path['alias'])) {
    drupal_set_message(t('You cannot lock an empty alias!'), 'warning');
  }
  $locked = alias_lock_load($path['pid']);
  $is_changed = $locked['alias'] !== $path['alias'];
  if ($is_changed) {
    if (!alias_lock_load($path['pid'])) {
      drupal_write_record('alias_lock', $path);
    }
    else {
      drupal_write_record('alias_lock', $path, array('pid'));
    }

    drupal_set_message(t("The alias for %source is now locked as: %alias", array(
      '%source' => $path['source'],
      '%alias' => $path['alias'],
    )), 'status', FALSE);

    // Clear the static alias cache.
    drupal_static_reset('alias_lock_load');
    drupal_clear_path_cache($path['source']);

    return ALIAS_LOCK_LOCKED;
  }
}

/**
 * Unlock an alias.
 *
 * @param array $path
 *   A number representing the pid or an array of criteria.
 *
 * @return null|ALIAS_LOCK_UNLOCKED
 *   ALIAS_LOCK_UNLOCKED is returned if the alias was previously locked and is
 *   now unlocked.
 */
function alias_lock_unlock(array $path) {
  if (!($locked = alias_lock_load($path['pid']))) {
    return;
  }
  $query = db_delete('alias_lock');
  foreach (element_children($locked) as $field) {
    $value = $locked[$field];
    $query->condition($field, $value);
  }
  $query->execute();

  drupal_set_message(t("The alias for %source has been unlocked.", array(
    '%source' => $path['source'],
    '%alias' => $path['alias'],
  )), 'status', FALSE);

  drupal_static_reset('alias_lock_load');
  drupal_clear_path_cache($path['source']);

  return ALIAS_LOCK_UNLOCKED;
}

/**
 * Handle the toggle of a lock on a node.
 *
 * @param array $path
 *   The path definition array.
 */
function _alias_lock_toggle_lock(array $path) {
  if (isset($path) && isset($path['pid']) && isset($path['alias_lock'])) {
    $locked = alias_lock_load($path['pid']);
    $needs_update = boolval($path['alias_lock']) !== boolval($locked);
    if ($needs_update) {
      if ($path['alias_lock']) {
        alias_lock_lock($path);
      }
      else {
        alias_lock_unlock($path);
      }
    }
  }
}

/**
 * Helper to alias_lock_form_path_admin_form_alter().
 */
function _alias_lock_form_path_admin_form_alter(&$form) {
  $source = isset($form['source']['#default_value']) ? $form['source']['#default_value'] : NULL;
  $locked = alias_lock_load($source);
  $is_locked = boolval($locked);
  $title = t('Lock alias from being deleted or updated?');
  $form['path']['alias_lock'] = array(
    '#type' => 'checkbox',
    '#title' => $title,
    '#default_value' => $is_locked,
    '#access' => user_access('lock url aliases') || user_access('administer url aliases'),
    '#attributes' => array(
      'data-alias-lock' => $locked['alias'],
      'class' => array(
        $is_locked ? 'is-locked' : '',
      ),
    ),
  );
  if ($is_locked) {
    $form['path']['alias_lock']['#suffix'] = t('<span class="js-alias-lock__disabled element-hidden"><strong>To lock another alias than %alias</strong>, you must first save this form, then return and lock it.</span>', array(
      '%alias' => $locked['alias'],
    ));
  }

  // Override path.module's and/or pathauto.module's vertical tabs summary.
  $form['path']['#attached']['js'] = array(
    'alias_lock' => drupal_get_path('module', 'alias_lock') . '/alias_lock.path.js',
  );

}

/**
 * Generate an augmented form of path_admin_overview page.
 *
 * @return array
 *   The build array.
 *
 * @see path_admin_overview
 */
function alias_lock_path_admin_overview() {
  module_load_include('inc', 'path', 'path.admin');
  $build = path_admin_overview();

  array_unshift($build['path_table']['#header'], t('Locked'));
  foreach ($build['path_table']['#rows'] as &$row) {
    $alias = strip_tags($row['data']['alias']);
    $locked = alias_lock_load(array('alias' => $alias));
    $row['data'] = array_merge(array('locked' => $locked ? '✓' : ''), $row['data']);

    // Remove the delete link.
    if ($locked) {
      unset($row['data']['operations']['data']['#links']['delete']);
    }
  }

  return $build;
}

/**
 * Implements hook_field_attach_form().
 */
function alias_lock_field_attach_form($entity_type, $entity, &$form) {
  if (!isset($form['path'])) {
    // This entity must be supported by core's path.module first.
    // @todo Investigate removing this and supporting all fieldable entities.
    return;
  }
  else {
    $pid = (int) $form['path']['pid']['#value'];
    $locked = alias_lock_load($pid);
    $is_locked = boolval($locked);
    $title = t('Lock alias from being deleted or updated?');
    $form['path']['alias_lock'] = array(
      '#type' => 'checkbox',
      '#title' => $title,
      '#default_value' => $is_locked,
      '#access' => user_access('lock url aliases') || user_access('administer url aliases'),
      '#attributes' => array(
        'data-alias-lock' => $locked['alias'],
        'class' => array(
          $is_locked ? 'is-locked' : '',
        ),
      ),
    );
    if ($is_locked) {
      $form['path']['alias_lock']['#suffix'] = t('<span class="js-alias-lock__disabled element-hidden"><strong>To lock another alias than %alias</strong>, you must first save this form, then return and lock it.</span>', array(
        '%alias' => $locked['alias'],
      ));
    }

    // Override path.module's and/or pathauto.module's vertical tabs summary.
    $form['path']['#attached']['js'] = array(
      'vertical-tabs' => drupal_get_path('module', 'alias_lock') . '/alias_lock.js',
    );
  }
}


/**
 * Helper function.
 *
 * @see alias_lock_lock_entity_action
 * @see alias_lock_unlock_entity_action
 */
function _alias_lock_entity_action($op, $entity, $entity_type) {
  $uri = entity_uri($entity_type, $entity);
  $path = path_load($uri['path']);
  if ($op === 'lock') {
    alias_lock_lock($path);
  }
  elseif ($op === 'unlock') {
    alias_lock_unlock($path);
  }
}
